import os
from pygit2 import Repository, discover_repository, GitError # pylint: disable = no-name-in-module

class CurrentRepositoryFactory:

    def create(self):
        basePath = os.getcwd()
        repositoryPath = discover_repository(basePath)

        if not repositoryPath:
            raise GitError(f'No repository found at "{basePath}" and its parents')

        return Repository(repositoryPath)



from argparse import ArgumentParser, Namespace
from logging import Logger
from box import Box
from consolebundle.ConsoleCommand import ConsoleCommand
from jobsbundle.job.ValuesFiller import ValuesFiller
from databricks_api.databricks import DatabricksAPI
from jobsbundle.job.JobPermissionUpdater import JobPermissionUpdater


class StreamingJobCreateCommand(ConsoleCommand):

    def __init__(
        self,
        jobsRawConfig: Box,
        logger: Logger,
        dbxApi: DatabricksAPI,
        valuesFiller: ValuesFiller,
        permissionUpdater: JobPermissionUpdater
    ):
        self.__jobsRawConfig = jobsRawConfig
        self.__logger = logger
        self.__dbxApi = dbxApi
        self.__valuesFiller = valuesFiller
        self.__permissionUpdater = permissionUpdater

    def getCommand(self) -> str:
        return 'databricks:job:streaming-create'

    def getDescription(self):
        return 'Creates new Databricks streaming job based on given job identifier'

    def configure(self, argumentParser: ArgumentParser):
        argumentParser.add_argument(dest='identifier', help='Job identifier')

    def run(self, inputArgs: Namespace):
        if inputArgs.identifier not in self.__jobsRawConfig:
            self.__logger.error('No job found for {}. Maybe you forgot to add the configuration under jobsbundle.jobs?'.format(inputArgs.identifier))
            return

        jobRawConfig = self.__jobsRawConfig[inputArgs.identifier].to_dict()
        jobConfig = self.__valuesFiller.fill(
            jobRawConfig['template'],
            jobRawConfig['values'],
            inputArgs.identifier
        )

        jobId = self.__dbxApi.jobs.create_job(**jobConfig.to_dict())['job_id']
        self.__logger.info(f'Job with ID {jobId} successfully created')

        self.__dbxApi.jobs.run_now(jobId)
        self.__logger.info(f'Job with ID {jobId} successfully run')

        if 'permission' in jobRawConfig:
            self.__permissionUpdater.run(jobRawConfig['permission'], jobId)



import sys
from logging import Logger
import requests
import json


class JobPermissionUpdater:

    def __init__(
        self,
        logger: Logger,
        databricksHost: str,
        databricksToken: str,
    ):
        self.__logger = logger
        self.__host = databricksHost
        self.__token = databricksToken

    def run(self, configPermission, jobId):
        resOfChange = self.__changePermissions(configPermission, jobId)

        if resOfChange.status_code != 200:
            self.__logger.error(f'Permissions for job ID {jobId} were not changed')
            self.__logger.error(resOfChange.text)
            sys.exit(1)

        self.__logger.info(f'Permissions for job ID {jobId} updated')
        resOfCheck = self.__checkPermissions(jobId)
        self.__logger.info(f'Current permissions: {resOfCheck.text}')

    def __changePermissions(self, configPermission, jobId):
        self.__logger.info(f'Changing permissions for job ID {jobId}')
        url = self.__createUrl(jobId)
        auth = self.__createAuth()

        data = {
            "object_id": f"/jobs/{jobId}",
            "object_type": "job",
            "access_control_list": self.__createAccessControlList(configPermission)
        }

        return requests.patch(url, data=json.dumps(data), headers=auth)

    def __createAccessControlList(self, configPermission):
        if 'usersNames' in configPermission:
            return [{'user_name': userName, 'permission_level': configPermission['permissionLevel']} for userName in configPermission['usersNames']]

        if 'groups' in configPermission:
            return [{'group_name': groupName, 'permission_level': configPermission['permissionLevel']} for groupName in configPermission['groups']]

        raise Exception(f'Invalid permissions: {configPermission}')

    def __checkPermissions(self, jobId):
        self.__logger.info(f'Checking the results after update')
        url = self.__createUrl(jobId)
        auth = self.__createAuth()
        return requests.get(url, headers=auth)

    def __createUrl(self, jobId):
        return f'{self.__host}/api/2.0/preview/permissions/jobs/{jobId}'

    def __createAuth(self):
        return {'Authorization': f'Bearer {self.__token}'}



import sys
from argparse import ArgumentParser, Namespace
from logging import Logger
from box import Box
from consolebundle.ConsoleCommand import ConsoleCommand
from jobsbundle.job.ValuesFiller import ValuesFiller
from jobsbundle.job.JobIdFinder import JobIdFinder
from jobsbundle.job.JobPermissionUpdater import JobPermissionUpdater

class JobPermissionUpdaterCommand(ConsoleCommand):

    def __init__(
        self,
        jobsRawConfig: Box,
        logger: Logger,
        valuesFiller: ValuesFiller,
        jobIdFinder: JobIdFinder,
        permissionUpdater: JobPermissionUpdater
    ):
        self.__jobsRawConfig = jobsRawConfig
        self.__logger = logger
        self.__valuesFiller = valuesFiller
        self.__jobIdFinder = jobIdFinder
        self.__permissionUpdater = permissionUpdater

    def getCommand(self) -> str:
        return 'databricks:job:set-permissions'

    def getDescription(self):
        return 'Update permissions for an existing Databricks job based on given job identifier'

    def configure(self, argumentParser: ArgumentParser):
        argumentParser.add_argument(dest='identifier', help='Job identifier')

    def run(self, inputArgs: Namespace):
        if inputArgs.identifier not in self.__jobsRawConfig:
            self.__logger.error('No job found for {}. Maybe you forgot to add the configuration under jobsbundle.jobs?'.format(inputArgs.identifier))
            sys.exit(1)

        jobRawConfig = self.__jobsRawConfig[inputArgs.identifier].to_dict()
        jobConfig = self.__valuesFiller.fill(
            jobRawConfig['template'],
            jobRawConfig['values'],
            inputArgs.identifier
        )
        self.__logger.info(f'Looking for job with name "{jobConfig.name}"')

        jobId = self.__jobIdFinder.find(jobConfig.name)

        if not jobId:
            self.__logger.error(f'No existing job with name "{jobConfig.name}" found')
            sys.exit(1)

        if 'permission' not in jobRawConfig:
            self.__logger.error(f'No permissions set in config')
            sys.exit(1)

        self.__permissionUpdater.run(jobRawConfig['permission'], jobId)



from jobsbundle.git.CurrentRepositoryFactory import CurrentRepositoryFactory

class CurrentBranchResolver:

    def __init__(
        self,
        currentRepositoryFactory: CurrentRepositoryFactory,
    ):
        self.__currentRepositoryFactory = currentRepositoryFactory

    def resolve(self) -> str:
        return self.__currentRepositoryFactory.create().head.shorthand



from logging import Logger
from databricks_api.databricks import DatabricksAPI


class JobIdFinder:

    def __init__(
        self,
        logger: Logger,
        dbxApi: DatabricksAPI,
    ):
        self.__logger = logger
        self.__dbxApi = dbxApi

    def find(self, jobName: str):
        jobsResponse = self.__dbxApi.jobs.list_jobs()

        if 'jobs' not in jobsResponse:
            self.__logger.info('No jobs exist')
            return None

        jobs = jobsResponse['jobs']
        for job in jobs:
            if job['settings']['name'] == jobName:
                return job['job_id']
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['jobsbundle']

package_data = \
{'': ['*'], 'jobsbundle': ['_config/*', 'git/*', 'job/*']}

install_requires = \
['console-bundle>=0.2.0,<0.3.0',
 'databricks-api>=0.3.0,<0.4.0',
 'injecta>=0.7',
 'pyfony-bundles>=0.2.0,<0.3.0',
 'pygit2>=1.3.0,<1.4.0']

setup_kwargs = {
    'name': 'jobs-bundle',
    'version': '0.2.3b1',
    'description': 'Databricks jobs management bundle for the Bricksflow Framework',
    'long_description': '# Databricks jobs bundle\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bricksflow/jobs-bundle',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.3,<3.8.0',
}


setup(**setup_kwargs)
